
//   Copyright 2020 Esri
//   Licensed under the Apache License, Version 2.0 (the "License");
//   you may not use this file except in compliance with the License.
//   You may obtain a copy of the License at
//
//   https://www.apache.org/licenses/LICENSE-2.0
//
//   Unless required by applicable law or agreed to in writing, software
//   distributed under the License is distributed on an "AS IS" BASIS,
//   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//   See the License for the specific language governing permissions and
//   limitations under the License.
//

#include "Display_a_map.h"
#include "ArcGISRuntimeEnvironment.h"

#include "Basemap.h"

#include "Map.h"
#include "MapQuickView.h"

#include "GraphicsOverlay.h"
#include "PolylineBuilder.h"
#include "PolygonBuilder.h"
#include "SimpleMarkerSymbol.h"
#include "SimpleFillSymbol.h"
#include "SimpleFillSymbol.h"

#include <QUrl>

using namespace Esri::ArcGISRuntime;

Display_a_map::Display_a_map(QObject* parent /* = nullptr */):
    QObject(parent),
    m_map(new Map(BasemapStyle::ArcGISTopographic, this))
{
}

Display_a_map::~Display_a_map()
{
}

MapQuickView* Display_a_map::mapView() const
{
    return m_mapView;
}

// Set the view (created in QML)

void Display_a_map::setMapView(MapQuickView* mapView)
{
  if (!mapView || mapView == m_mapView)
  {
    return;
  }

  m_mapView = mapView;
  m_mapView->setMap(m_map);
  setupViewpoint();

  GraphicsOverlay* overlay = new GraphicsOverlay(this);
  createGraphics(overlay);
  m_mapView->graphicsOverlays()->append(overlay);

  emit mapViewChanged();
}

void Display_a_map::setupViewpoint()
{
  const Point center(-118.80543, 34.02700, SpatialReference::wgs84());
  const Viewpoint viewpoint(center, 100000.0);
  m_mapView->setViewpoint(viewpoint);
}

void Display_a_map::createGraphics(GraphicsOverlay *overlay)
{

  // Create a point
  const Point dume_beach(-118.80657463861, 34.0005930608889, SpatialReference::wgs84());
  // Create symbols for the point
  SimpleLineSymbol* point_outline = new SimpleLineSymbol(SimpleLineSymbolStyle::Solid, QColor(Qt::blue), 3, this);
  SimpleMarkerSymbol* point_symbol = new SimpleMarkerSymbol(SimpleMarkerSymbolStyle::Circle, QColor(Qt::red), 10, this);
  point_symbol->setOutline(point_outline);

  // Create a graphic to display the point with its symbology
  Graphic* point_graphic = new Graphic(dume_beach, point_symbol, this);
  // Add point graphic to the graphics overlay
  overlay->graphics()->append(point_graphic);

  // Create a line
  PolylineBuilder* polyline_builder = new PolylineBuilder(SpatialReference::wgs84(), this);
  polyline_builder->addPoint(-118.8215, 34.0140);
  polyline_builder->addPoint(-118.8149, 34.0081);
  polyline_builder->addPoint(-118.8089, 34.0017);
  // Create a symbol for the line
  SimpleLineSymbol* line_symbol = new SimpleLineSymbol(SimpleLineSymbolStyle::Solid, QColor(Qt::blue), 3, this);

  // Create a graphic to display the line with its symbology
  Graphic* polyline_graphic = new Graphic(polyline_builder->toGeometry(), line_symbol, this);
  // Add line graphic to the graphics overlay
  overlay->graphics()->append(polyline_graphic);

  // Create a list of points to make up the polygon
  const QList<Point> points = {
    Point(-118.8190, 34.0138),
    Point(-118.8068, 34.0216),
    Point(-118.7914, 34.0164),
    Point(-118.7960, 34.0086),
    Point(-118.8086, 34.0035),
  };
  // Create a polygon using the list of points above
  PolygonBuilder* polygon_builder = new PolygonBuilder(SpatialReference::wgs84(), this);
  polygon_builder->addPoints(points);
  // Create symbols for the polygon
  SimpleLineSymbol* polygon_line_symbol = new SimpleLineSymbol(SimpleLineSymbolStyle::Solid, QColor(Qt::blue), 3, this);
  SimpleFillSymbol* fill_symbol = new SimpleFillSymbol(SimpleFillSymbolStyle::Solid, QColor(Qt::yellow), polygon_line_symbol, this);
  // Create a graphic to display the polygon with its symbology
  Graphic* polygon_graphic = new Graphic(polygon_builder->toGeometry(), fill_symbol, this);
  // Add polygon graphic to the graphics overlay
  overlay->graphics()->append(polygon_graphic);

}


